###############   PSQF 6271 Longitudinal MLM Example 6 using R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=6) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do linear combinations in gls

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC in lmer

if (!require("margins")) install.packages("margins")
library(margins) # To get predicted values like Stata does

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit path and SEMs 


################################################################################
#####           BEGIN DATA MANIPULATION OF CHAPTER 6 EXAMPLE               #####
#####            CHANGE "filesave" TO YOUR DIRECTORY                       #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6271/PSQF6271_Example6/"
filename = "SAS_Chapter6.sas7bdat"
setwd(dir=filesave)

# Import chapter 6 stacked data with labels
Example6 = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example6 = as.data.frame(Example6)
# Sort data by PersonID (needed for correct RCOV matrix)
Example6 = sort_asc(Example6,PersonID,session)

# Make new variable for session with reference=6 to match other programs
Example6$session6=relevel(factor(Example6$session), ref=6)

# Center time predictor for polynomial time models
Example6$time=Example6$session-1

# Create session dummy codes for testing means model absolute fit in REML
Example6$s1=0
Example6$s2=0
Example6$s3=0
Example6$s4=0
Example6$s5=0
Example6$s6=0
Example6$s1[which(Example6$session==1)]=1
Example6$s2[which(Example6$session==2)]=1
Example6$s3[which(Example6$session==3)]=1
Example6$s4[which(Example6$session==4)]=1
Example6$s5[which(Example6$session==5)]=1
Example6$s6[which(Example6$session==6)]=1

# Create predictors for piecewise time models assuming BALANCED DATA
# slope12 and slope26 for 2 Direct Slopes model, time for Slope + Deviation Slope model
# Both models have intercept at session 1, breakpoint at session 2
Example6$slope12=Example6$session
Example6$slope12[which(Example6$session==1)]=0
Example6$slope12[which(Example6$session==2)]=1
Example6$slope12[which(Example6$session==3)]=1
Example6$slope12[which(Example6$session==4)]=1
Example6$slope12[which(Example6$session==5)]=1
Example6$slope12[which(Example6$session==6)]=1
Example6$slope26=Example6$session
Example6$slope26[which(Example6$session==1)]=0
Example6$slope26[which(Example6$session==2)]=0
Example6$slope26[which(Example6$session==3)]=1
Example6$slope26[which(Example6$session==4)]=2
Example6$slope26[which(Example6$session==5)]=3
Example6$slope26[which(Example6$session==6)]=4

# Log-transform time as new predictor
Example6$logtime=log(Example6$session)


################################################################################
#####           DESCRIPTIVE AND POLYNOMIAL MODELS                          #####
################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example6_Output_A.Polynomial.txt"))

print("Chapter 6 Example: Means by session for RT outcome")
describeBy(x=Example6$rt, digits=2, group=Example6$session)  

txtComment(" ") # insert blank space in output

print("R Ch 6: 0: Saturated Means, Unstructured Variance Model in GLS -- TOTAL ANSWER KEY")
SatUN = gls(data=Example6, method="REML", model=rt~1+factor(session6), 
            correlation=corSymm(form=~as.numeric(session)|PersonID), # Unstructured correlations
            weights=varIdent(form=~1|session))                       # Heterogeneous variances
print("Show results with -2LL, incorrect DDF, and total leftover variance")
print("Total variance per occasion is created using SD multiplier")
summary(SatUN); -2*logLik(SatUN); summary(SatUN)$sigma^2 
print("Show R and RCORR matrices for first person (R is slightly off)") 
R=getVarCov(SatUN, individual="101", type="marginal"); R
RCORR=corMatrix(SatUN$modelStruct$corStruct)[[4]]; RCORR 
print("session means and pairwise mean differences with incorrect DDF")
emmeans(ref_grid(SatUN), pairwise~session6, adjust="none") # tried mode="df.error"
print("Error when trying to get Satterthwaite DDF, so had to switch to residual DDF")
lsmeans(SatUN, "session6", mode="df.error")
print("F-test p-value now based on Satterthwaite DDF"); anova(SatUN)

txtComment(" ") # insert blank space in output

print("R Ch 6: 1b: Empty Means, Random Intercept Model")
EmptyRI = lmer(data=Example6, REML=TRUE, formula=rt~1+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(EmptyRI, ddf="Satterthwaite"); llikAIC(EmptyRI, chkREML=FALSE)
print("Get ICC"); icc(EmptyRI)
print("Does random intercept improve model fit?")
ranova(EmptyRI, reduce.term=TRUE) # Remove random intercept

txtComment(" ") # insert blank space in output

print("R Ch 6 2a: Fixed Linear Time, Random Intercept Model")
FixLin = lmer(data=Example6, REML=TRUE, formula=rt~1+time+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(FixLin, ddf="Satterthwaite"); llikAIC(FixLin, chkREML=FALSE)
print("Get conditional mean per occasion from value of time predictor")
print("Intercept at Session=1 Time=0"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,0)) 
print("Intercept at Session=2 Time=1"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,1)) 
print("Intercept at Session=3 Time=2"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,2))  
print("Intercept at Session=4 Time=3"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,3))
print("Intercept at Session=5 Time=4"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,4))
print("Intercept at Session=6 Time=5"); contest1D(FixLin, ddf="Satterthwaite", L=c(1,5))

txtComment(" ") # insert blank space in output

print("R Ch 6 2b: Random Linear Time Model")
RandLin = lmer(data=Example6, REML=TRUE, formula=rt~1+time+(1+time|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(RandLin, ddf="Satterthwaite"); llikAIC(RandLin, chkREML=FALSE)
print("Get conditional mean per occasion from value of time predictor")
print("Intercept at Session=1 Time=0"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,0)) 
print("Intercept at Session=2 Time=1"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,1)) 
print("Intercept at Session=3 Time=2"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,2))  
print("Intercept at Session=4 Time=3"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,3))
print("Intercept at Session=5 Time=4"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,4))
print("Intercept at Session=6 Time=5"); contest1D(RandLin, ddf="Satterthwaite", L=c(1,5))
print("Does random linear time slope improve model fit?")
ranova(RandLin, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

print("R Ch 6 3a: Fixed Quadratic, Random Linear Time Model")
FixQuad = lmer(data=Example6, REML=TRUE, formula=rt~1+time+I(time^2)+(1+time|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(FixQuad, ddf="Satterthwaite"); llikAIC(FixQuad, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors")
print("Intercept at Session=1 Time=0"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,1,1)) 
print("Intercept at Session=3 Time=2"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,2,4))  
print("Intercept at Session=4 Time=3"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,3,9))
print("Intercept at Session=5 Time=4"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,4,16))
print("Intercept at Session=6 Time=5"); contest1D(FixQuad, ddf="Satterthwaite", L=c(1,5,25))
print("Get instantaneous linear slope per occasion from 2*value of time predictor")
print("Linear Slope at Session=1 Time=0"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,0)) 
print("Linear Slope at Session=2 Time=1"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,2)) 
print("Linear Slope at Session=3 Time=2"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,4)) 
print("Linear Slope at Session=4 Time=3"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,6)) 
print("Linear Slope at Session=5 Time=4"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,8)) 
print("Linear Slope at Session=6 Time=5"); contest1D(FixQuad, ddf="Satterthwaite", L=c(0,1,10)) 

txtComment(" ") # insert blank space in output

print("R Ch 6 3b: Random Quadratic Time Model -- reports convergence problem")
RandQuad = lmer(data=Example6, REML=TRUE, formula=rt~1+time+I(time^2)+(1+time+I(time^2)|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(RandQuad, ddf="Satterthwaite"); llikAIC(RandQuad, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors")
print("Intercept at Session=1 Time=0"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,1,1)) 
print("Intercept at Session=3 Time=2"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,2,4))  
print("Intercept at Session=4 Time=3"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,3,9))
print("Intercept at Session=5 Time=4"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,4,16))
print("Intercept at Session=6 Time=5"); contest1D(RandQuad, ddf="Satterthwaite", L=c(1,5,25))
print("Get instantaneous linear slope per occasion from 2*value of time predictor")
print("Linear Slope at Session=1 Time=0"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,0)) 
print("Linear Slope at Session=2 Time=1"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,2)) 
print("Linear Slope at Session=3 Time=2"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,4)) 
print("Linear Slope at Session=4 Time=3"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,6)) 
print("Linear Slope at Session=5 Time=4"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,8)) 
print("Linear Slope at Session=6 Time=5"); contest1D(RandQuad, ddf="Satterthwaite", L=c(0,1,10)) 
print("Does random quadratic time slope improve model fit?")
ranova(RandQuad, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

print("R: Test AR1 Residual Correlation in Random Quadratic Time Model in LME")
RandQuadAR1 = lme(data=Example6, method="REML", rt~1+time+I(time^2), random=~1+time+I(time^2)|PersonID,
                  correlation=(corAR1(form=~as.numeric(time)|PersonID)))
print("Show results using incorrect DDF"); summary(RandQuadAR1)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(RandQuadAR1, individual="101", type="random.effects"); G
R=getVarCov(RandQuadAR1, individual="101", type="conditional"); R
V=getVarCov(RandQuadAR1, individual="101", type="marginal"); V
print("Does AR1 residual correlation improve fit?")
print("Have to re-run random quadratic time model using lme to get LRT")
RandQuadlme = lme(data=Example6, method="REML", rt~1+time+I(time^2), 
                  random=~1+time+I(time^2)|PersonID)
anova(RandQuadAR1,RandQuadlme) # anova does LRT using LME versions

txtComment(" ") # insert blank space in output

print("R: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)")
print("Add 3 session dummy codes to saturate the means model")
QuadMean = lmer(data=Example6, REML=TRUE, formula=rt~1+time+I(time^2)+s1+s2+s3+(1+time+I(time^2)|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(QuadMean, ddf="Satterthwaite"); llikAIC(QuadMean, chkREML=FALSE)
print("Does fixed quadratic time reproduce saturated means?")
contestMD(QuadMean, ddf="Satterthwaite", L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output                                            

print("R: Test Absolute Fit of Quadratic Time Means Model (Using Random Quadratic Variance Model)")
print("Add different 3 session dummy codes to saturate the means model")
QuadMean2 = lmer(data=Example6, REML=TRUE, formula=rt~1+time+I(time^2)+s4+s5+s6+(1+time+I(time^2)|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(QuadMean2, ddf="Satterthwaite"); llikAIC(QuadMean2, chkREML=FALSE)
print("Does fixed quadratic time reproduce saturated means?")
contestMD(QuadMean2, ddf="Satterthwaite", L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output    

print("R: Testing Absolute Fit of Quadratic Time Variance Model (Using Fixed Quadratic Means Model)")
print("Change to Unstructured R matrix as variance model answer key in GLS")
FixQuadUN = gls(data=Example6, method="REML", model=rt~1+time+I(time^2), 
                correlation=corSymm(form=~as.numeric(session)|PersonID),  # Unstructured correlations
                weights=varIdent(form=~1|session))                        # Heterogeneous variances 
print("Have to re-run random quadratic time model using LME to get LRT")
RandQuadlme = lme(data=Example6, method="REML", rt~1+time+I(time^2), 
                  random=~1+time+I(time^2)|PersonID)
anova(FixQuadUN,RandQuadlme) # anova does LRT using LME versions

txtComment(" ") # insert blank space in output

# Close external file of saved results
txtStop()


################################################################################
#####                      PIECEWISE MODELS                                #####
################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example6_Output_B.Piecewise.txt"))

print("R Ch 6 4a: Fixed Slope12, Fixed Slope26, Random Intercept Model")
Fix12Fix26 = lmer(data=Example6, REML=TRUE, formula=rt~1+slope12+slope26+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Fix12Fix26, ddf="Satterthwaite"); llikAIC(Fix12Fix26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictor, also slope difference")
print("Intercept at Session=1 Time=0"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,1,1))  
print("Intercept at Session=4 Time=3"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,1,2))
print("Intercept at Session=5 Time=4"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,1,3))
print("Intercept at Session=6 Time=5"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(1,1,4))
print("Difference of fixed slope12 vs slope26"); contest1D(Fix12Fix26, ddf="Satterthwaite", L=c(0,-1,1))

txtComment(" ") # insert blank space in output    

print("R Ch 6 4b: Random Slope12, Fixed Slope26 Model")
Rand12Fix26 = lmer(data=Example6, REML=TRUE, formula=rt~1+slope12+slope26+(1+slope12|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Rand12Fix26, ddf="Satterthwaite"); llikAIC(Rand12Fix26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors, also slope difference")
print("Intercept at Session=1 Time=0"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,1,1))  
print("Intercept at Session=4 Time=3"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,1,2))
print("Intercept at Session=5 Time=4"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,1,3))
print("Intercept at Session=6 Time=5"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(1,1,4))
print("Difference of fixed slope12 vs slope26"); contest1D(Rand12Fix26, ddf="Satterthwaite", L=c(0,-1,1))
print("Does random slope12improve model fit?")
ranova(Rand12Fix26, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output    

print("R Ch 6 4c: Random Slope12, Random Slope26 Model -- reports convergence problem")
Rand12Rand26 = lmer(data=Example6, REML=TRUE, formula=rt~1+slope12+slope26+(1+slope12+slope26|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Rand12Rand26, ddf="Satterthwaite"); llikAIC(Rand12Rand26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors, also slope difference")
print("Intercept at Session=1 Time=0"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,1,1))  
print("Intercept at Session=4 Time=3"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,1,2))
print("Intercept at Session=5 Time=4"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,1,3))
print("Intercept at Session=6 Time=5"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(1,1,4))
print("Difference of fixed slope12 vs slope26"); contest1D(Rand12Rand26, ddf="Satterthwaite", L=c(0,-1,1))
print("Does random slope26 improve model fit?")
ranova(Rand12Rand26, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output    

print("R Ch 6 5a: Fixed Time, Fixed Slope26, Random Intercept Model")
Fix16Fix26 = lmer(data=Example6, REML=TRUE, formula=rt~1+time+slope26+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Fix16Fix26, ddf="Satterthwaite"); llikAIC(Fix12Fix26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors, also slope26")
print("Intercept at Session=1 Time=0"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,2,1))  
print("Intercept at Session=4 Time=3"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,3,2))
print("Intercept at Session=5 Time=4"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,4,3))
print("Intercept at Session=6 Time=5"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(1,5,4))
print("Fixed rate of change from session 2 to 6"); contest1D(Fix16Fix26, ddf="Satterthwaite", L=c(0,1,1))

txtComment(" ") # insert blank space in output    

print("R Ch 6 5b: Random Time, Fixed Slope26 Model")
Rand16Fix26 = lmer(data=Example6, REML=TRUE, formula=rt~1+time+slope26+(1+time|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Rand16Fix26, ddf="Satterthwaite"); llikAIC(Rand16Fix26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors, also slope26")
print("Intercept at Session=1 Time=0"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,2,1))  
print("Intercept at Session=4 Time=3"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,3,2))
print("Intercept at Session=5 Time=4"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,4,3))
print("Intercept at Session=6 Time=5"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(1,5,4))
print("Fixed rate of change from session 2 to 6"); contest1D(Rand16Fix26, ddf="Satterthwaite", L=c(0,1,1))
print("Does random time slope improve model fit?")
ranova(Rand16Fix26, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output    

print("R Ch 6 5c: Random Time, Random Slope26 Model")
Rand16Rand26 = lmer(data=Example6, REML=TRUE, formula=rt~1+time+slope26+(1+time+slope26|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(Rand16Rand26, ddf="Satterthwaite"); llikAIC(Rand16Rand26, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors, also slope26")
print("Intercept at Session=1 Time=0"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2 Time=1"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,1,0)) 
print("Intercept at Session=3 Time=2"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,2,1))  
print("Intercept at Session=4 Time=3"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,3,2))
print("Intercept at Session=5 Time=4"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,4,3))
print("Intercept at Session=6 Time=5"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(1,5,4))
print("Fixed rate of change from session 2 to 6"); contest1D(Rand16Rand26, ddf="Satterthwaite", L=c(0,1,1))
print("Does random slope26 improve model fit?")
ranova(Rand16Rand26, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

print("R: Test Absolute Fit of Two-Piece Means Model (Using Random Two-Piece Variance Model)")
print("Add 3 session dummy codes to SECOND PIECE to saturate the means model")
PieceMean = lmer(data=Example6, REML=TRUE, formula=rt~1+time+slope26+s4+s5+s6+(1+time+slope26|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(PieceMean, ddf="Satterthwaite"); llikAIC(PieceMean, chkREML=FALSE)
print("Does fixed two-piece slope reproduce saturated means?")
contestMD(PieceMean, ddf="Satterthwaite", L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output    

print("R: Test Absolute Fit of Two-Piece Variance Model (Using Fixed Two-Piece Means Model)")
print("Change to Unstructured R matrix as variance model answer key in GLS")
FixPieceUN = gls(data=Example6, method="REML", model=rt~1+time+slope26, 
                 correlation=corSymm(form=~as.numeric(session)|PersonID),  # Unstructured correlations
                 weights=varIdent(form=~1|session))                        # Heterogeneous variances 
print("Have to re-run random two-piece model using LME to get LRT")
RandPiecelme = lme(data=Example6, method="REML", rt~1+time+slope26, random=~1+time+slope26|PersonID)
anova(FixPieceUN,RandPiecelme) # anova does LRT using LME versions

txtComment(" ") # insert blank space in output


# Close external file of saved results
txtStop()


################################################################################
#####             LATENT BASIS MODEL FOR EXAMPLE 6                         #####
################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example6_Output_c.LatentBasis.txt"))

# Convert from long to wide for SEM
Example6wide = reshape(data=Example6, direction="wide",v.names="rt", 
                       idvar=c("PersonID"), timevar="time")

# Had to rescale variables to get a positive definite solution
Example6wide$rt.0=Example6wide$rt.0/100
Example6wide$rt.1=Example6wide$rt.1/100
Example6wide$rt.2=Example6wide$rt.2/100
Example6wide$rt.3=Example6wide$rt.3/100
Example6wide$rt.4=Example6wide$rt.4/100
Example6wide$rt.5=Example6wide$rt.5/100

LatentBasisSyntax = "
# Factor loadings fixed by *  adding start values from Mplus
Int =~ 1*rt.0 + 1*rt.1 + 1*rt.2 + 1*rt.3 +  1*rt.4 +  1*rt.5
Slp =~ 0*rt.0 + start(.57)*rt.1 + start(.76)*rt.2 + start(.90)*rt.3 + start(.98)*rt.4 + 1*rt.5

# Factor intercepts estimated = fixed effects
  Int ~ 1; Slp ~ 1
# Level-2 factor variances estimated (in G)
  Int ~~ Int; Slp ~~ Slp
# Level-2 factor covariances estimated (in G)
  Int ~~ Slp
  
# Per-occasion intercepts fixed to 0
  rt.0 ~ 0; rt.1 ~ 0; rt.2 ~ 0; rt.3 ~ 0; rt.4 ~ 0; rt.5 ~ 0

! Level-1 residual variances held equal (in R) 
  rt.0 ~~ (ResVar)*rt.0;  rt.1 ~~ (ResVar)*rt.1
  rt.2 ~~ (ResVar)*rt.2;  rt.3 ~~ (ResVar)*rt.3
  rt.4 ~~ (ResVar)*rt.4;  rt.5 ~~ (ResVar)*rt.5
"
Model8 = lavaan(data=Example6wide, model=LatentBasisSyntax, estimator="ML", mimic="mplus")
summary(Model8, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Close external file of saved results
txtStop()


################################################################################
#####           EXTRA LOG TIME MODELS FOR EXAMPLE 6                        #####
################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example6_Output_D.LogTime.txt"))

print("R 7a: Fixed Linear Log Time, Random Intercept Model")
FixLinLog = lmer(data=Example6, REML=TRUE, formula=rt~1+logtime+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(FixLinLog, ddf="Satterthwaite"); llikAIC(FixLinLog, chkREML=FALSE)
print("Get conditional mean per occasion from value of time predictor")
print("Intercept at Session=1"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,0)) 
print("Intercept at Session=2"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,0.6931)) 
print("Intercept at Session=3"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,1.0986))  
print("Intercept at Session=4"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,1.3863))
print("Intercept at Session=5"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,1.6094))
print("Intercept at Session=6"); contest1D(FixLinLog, ddf="Satterthwaite", L=c(1,1.7918))

txtComment(" ") # insert blank space in output

print("R 7b: Random Linear Log Time Model")
RandLinLog = lmer(data=Example6, REML=TRUE, formula=rt~1+logtime+(1+logtime|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(RandLinLog, ddf="Satterthwaite"); llikAIC(RandLinLog, chkREML=FALSE)
print("Get conditional mean per occasion from value of time predictor")
print("Intercept at Session=1"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,0)) 
print("Intercept at Session=2"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,0.6931)) 
print("Intercept at Session=3"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,1.0986))  
print("Intercept at Session=4"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,1.3863))
print("Intercept at Session=5"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,1.6094))
print("Intercept at Session=6"); contest1D(RandLinLog, ddf="Satterthwaite", L=c(1,1.7918))
print("Does random linear time slope improve model fit?")
ranova(RandLinLog, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

print("R: Test Absolute Fit of Linear Log Time Means Model (Using Random Linear Log Time Variance Model)")
print("Add 4 session dummy codes to second piece to saturate the means model")
LinLogMean = lmer(data=Example6, REML=TRUE, formula=rt~1+logtime+s1+s2+s3+s4+(1+logtime|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(LinLogMean, ddf="Satterthwaite"); llikAIC(LinLogMean, chkREML=FALSE)
print("Does fixed two-piece slope reproduce saturated means?")
contestMD(LinLogMean, ddf="Satterthwaite", 
          L=rbind(c(0,0,1,0,0,0),c(0,0,0,1,0,0),c(0,0,0,0,1,0) ,c(0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output    

print("R: Test Absolute Fit of Linear Log Time Variance Model (Using Fixed Linear Log Time Means Model)")
print("Change to Unstructured R matrix as variance model answer key in GLS")
LinLogUN = gls(data=Example6, method="REML", model=rt~1+logtime, 
               correlation=corSymm(form=~as.numeric(session)|PersonID),  # Unstructured correlations
               weights=varIdent(form=~1|session))                        # Heterogeneous variances 
print("Have to re-run random quadratic log time model using LME to get LRT")
RandLinLoglme = lme(data=Example6, method="REML", rt~1+logtime, random=~1+logtime|PersonID)
anova(LinLogUN,RandLinLoglme) # anova does LRT using LME versions

txtComment(" ") # insert blank space in output

print("R 7c: Fixed Quadratic, Random Linear Log Time Model")
FixQuadLog = lmer(data=Example6, REML=TRUE, formula=rt~1+logtime+I(logtime^2)+(1+logtime|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(FixQuadLog, ddf="Satterthwaite"); llikAIC(FixQuadLog, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors")
print("Intercept at Session=1"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,0.6931,0.4805)) 
print("Intercept at Session=3"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,1.0986,1.2069))  
print("Intercept at Session=4"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,1.3863,1.9218))
print("Intercept at Session=5"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,1.6094,2.5903))
print("Intercept at Session=6"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(1,1.7918,3.2104))
print("Get instantaneous linear slope per occasion from 2*value of time predictor")
print("Linear Slope at Session=1 Time=0"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,0)) 
print("Linear Slope at Session=2 Time=1"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,1.3863)) 
print("Linear Slope at Session=3 Time=2"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,2.1972)) 
print("Linear Slope at Session=4 Time=3"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,2.7726)) 
print("Linear Slope at Session=5 Time=4"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,3.2189)) 
print("Linear Slope at Session=6 Time=5"); contest1D(FixQuadLog, ddf="Satterthwaite", L=c(0,1,3.5835)) 
txtComment(" ") # insert blank space in output

print("R 7d: Random Quadratic Log Time Model -- reports convergence problem")
RandQuadLog = lmer(data=Example6, REML=TRUE, formula=rt~1+logtime+I(logtime^2)+(1+logtime+I(logtime^2)|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(RandQuadLog, ddf="Satterthwaite"); llikAIC(RandQuadLog, chkREML=FALSE)
print("Get conditional mean per occasion from values of time predictors")
print("Intercept at Session=1"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,0,0)) 
print("Intercept at Session=2"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,0.6931,0.4805)) 
print("Intercept at Session=3"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,1.0986,1.2069))  
print("Intercept at Session=4"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,1.3863,1.9218))
print("Intercept at Session=5"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,1.6094,2.5903))
print("Intercept at Session=6"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(1,1.7918,3.2104))
print("Get instantaneous linear slope per occasion from 2*value of time predictor")
print("Linear Slope at Session=1 Time=0"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,0)) 
print("Linear Slope at Session=2 Time=1"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,1.3863)) 
print("Linear Slope at Session=3 Time=2"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,2.1972)) 
print("Linear Slope at Session=4 Time=3"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,2.7726)) 
print("Linear Slope at Session=5 Time=4"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,3.2189)) 
print("Linear Slope at Session=6 Time=5"); contest1D(RandQuadLog, ddf="Satterthwaite", L=c(0,1,3.5835)) 
print("Does random quadratic time slope improve model fit?")
ranova(RandQuadLog, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

print("R: Test Absolute Fit of Quadratic Log Time Means Model (Using Random Quadratic Log Time Variance Model)")
print("Add 3 session dummy codes to second piece to saturate the means model")
QuadLogMean = lmer(data=Example6, REML=TRUE, 
                   formula=rt~1+logtime+I(logtime^2)+s1+s2+s3+(1+logtime+I(logtime^2)|PersonID))
print("Show results using Satterthwaite DDF including -2LL as deviance") 
summary(QuadLogMean, ddf="Satterthwaite"); llikAIC(QuadLogMean, chkREML=FALSE)
print("Does fixed quadratic log time reproduce saturated means?")
contestMD(QuadLogMean, ddf="Satterthwaite", L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))
          
txtComment(" ") # insert blank space in output    

print("R: Test Absolute Fit of Quadratic Log Time Variance Model (Using Fixed Quadratic Log Time Means Model)")
print("Change to Unstructured R matrix as variance model answer key in GLS")
QuadLogUN = gls(data=Example6, method="REML", model=rt~1+logtime+I(logtime^2), 
                correlation=corSymm(form=~as.numeric(session)|PersonID),  # Unstructured correlations
                weights=varIdent(form=~1|session))                        # Heterogeneous variances 
print("Have to re-run random quadratic log time model using LME to get LRT")
RandQuadLoglme = lme(data=Example6, method="REML", rt~1+logtime+I(logtime^2), 
                     random=~1+logtime+I(logtime^2)|PersonID)
anova(QuadLogUN,RandQuadLoglme) # anova does LRT using LME versions

txtComment(" ") # insert blank space in output


################################################################################
#####     RE-RUNNING BEST VERSION OF EACH MODEL TO TEST ABSOLUTE FIT       #####
################################################################################

print("R Ch 6: 0: Saturated Means, Unstructured Variance Model using ML in GLS")
print("TOTAL ANSWER KEY")
SatUNML = gls(data=Example6, method="ML", model=rt~1+factor(session6), 
              correlation=corSymm(form=~as.numeric(session)|PersonID),
              weights=varIdent(form=~1|session)); summary(SatUNML)

txtComment(" ") # insert blank space in output

print("R Ch 6 3b: Random Quadratic Time Model using ML in LME")
RandQuadML = lme(data=Example6, method="ML", rt~1+time+I(time^2), 
                 random=~1+time+I(time^2)|PersonID); summary(RandQuadML, ddf="Satterthwaite")
print("Test absolute fit of random quadratic time model"); anova(SatUNML, RandQuadML)

txtComment(" ") # insert blank space in output

print("R Ch 6 4c: Random Slope12, Random Slope26 Model using ML in LME")
RandPieceML = lme(data=Example6, method="ML", rt~1+time+slope26, 
                  random=~1+time+slope26|PersonID); summary(RandPieceML, ddf="Satterthwaite")
print("Test absolute fit of random two-piece model"); anova(SatUNML, RandPieceML)

txtComment(" ") # insert blank space in output

print("R 7b: Random Linear Log Time Model using ML in LME")
RandLinLogML = lme(data=Example6, method="ML", rt~1+logtime, 
                   random=~1+logtime|PersonID); summary(RandLinLogML, ddf="Satterthwaite")
print("Test absolute fit of random linear log time model"); anova(SatUNML, RandLinLogML)

txtComment(" ") # insert blank space in output

print("R 7d: Random Quadratic Log Time Model using ML in LME")
RandQuadLogML = lme(data=Example6, method="ML", rt~1+logtime+I(logtime^2), 
                    random=~1+logtime+I(logtime^2)|PersonID); summary(RandQuadLogML, ddf="Satterthwaite")
print("Test absolute fit of random quadratic log time model"); anova(SatUNML, RandQuadLogML)

txtComment(" ") # insert blank space in output


# Close external file of saved results
txtStop()
